mod field;
mod pattern;
mod piece;

#[cfg(test)]
mod tests;

use field::Field;
use piece::Piece;

fn solve(field: Field, pieces: &mut Vec<Piece>) -> Option<Field> {
    if field.empty_coords.is_empty() {
        if pieces.is_empty() {
            return Some(field);
        } else {
            return None;
        }
    } else if pieces.is_empty() {
        return None;
    }

    for empty_coord in &field.empty_coords {
        for piece_index in 0..pieces.len() {
            let piece = pieces.remove(piece_index);
            for pattern in &piece.patterns {
                if field.fits(&empty_coord, &pattern) {
                    let mut new_field: Field = field.clone();
                    new_field.add_pattern(&empty_coord, &pattern);
                    if let Some(field) = solve(new_field, pieces) {
                        return Some(field);
                    }
                }
            }
            pieces.insert(piece_index, piece);
        }
    }
    None
}

fn main() {
    let field = Field::new(8, 7);

    // All pieces as reference:
    // Piece::cleveland_z(),     // -_
    // Piece::rhode_island_z(),  // _-
    // Piece::teewee(),          // _|_
    // Piece::hero(),            // ____
    // Piece::orange_ricky(),    // ___|
    // Piece::blue_ricky(),      // |___
    // Piece::smashboy(),        // o
    let mut pieces = vec![
        Piece::hero(), // ----
        Piece::hero(), // ----
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::cleveland_z(),    // -_
        Piece::rhode_island_z(), // _-
        Piece::orange_ricky(),   // ___|
        Piece::blue_ricky(),     // |___
    ];
    let result = solve(field, &mut pieces);
    match result {
        Some(field) => println!("{}", field),
        None => println!("No solution",),
    };
}
