use std::fmt;

#[derive(Debug, PartialEq, Clone)]
pub struct RelCoord {
    pub x: i32,
    pub y: i32,
}

#[derive(Debug, PartialEq, Clone)]
pub struct AbsCoord {
    pub x: u32,
    pub y: u32,
}

impl AbsCoord {
    pub fn add(&self, offset: &RelCoord) -> Option<Self> {
        Some(Self {
            x: ((self.x as i32) + offset.x).try_into().ok()?,
            y: ((self.y as i32) + offset.y).try_into().ok()?,
        })
    }
}

#[derive(Debug, PartialEq, Clone)]
pub struct Pattern {
    pub coords: Vec<RelCoord>,
}

impl fmt::Display for Pattern {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        // Determine the bounding box for the pattern.
        let max_x = self.coords.iter().map(|coord| coord.x).max().unwrap_or(0);
        let max_y = self.coords.iter().map(|coord| coord.y).max().unwrap_or(0);

        // Initialize a 2D grid to hold the pattern representation.
        let mut grid = vec![vec![' '; (max_x + 1) as usize]; (max_y + 1) as usize];

        // Place the blocks of the pattern onto the grid.
        for coord in &self.coords {
            grid[coord.y as usize][coord.x as usize] = '#';
        }

        // Convert the 2D grid to a string.
        for row in grid.iter() {
            for cell in row.iter() {
                write!(f, "{}", cell)?;
            }
            writeln!(f)?;
        }

        Ok(())
    }
}
