use crate::pattern::{Pattern, RelCoord};
use colored::Colorize;
use std::fmt;

#[derive(Debug, Clone)]
pub struct Piece {
    name: String,
    pub patterns: Vec<Pattern>,
}

impl Piece {
    #[allow(dead_code)]
    pub fn hero() -> Self {
        let coords_vert = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
            RelCoord { x: 0, y: 3 },
        ];
        let coords_hori = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 2, y: 0 },
            RelCoord { x: 3, y: 0 },
        ];
        Self {
            name: "Hero".to_string(),
            patterns: vec![
                Pattern {
                    coords: coords_vert,
                },
                Pattern {
                    coords: coords_hori,
                },
            ],
        }
    }

    #[allow(dead_code)]
    pub fn teewee() -> Self {
        let coords_up = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: -1, y: 1 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 1, y: 1 },
        ];
        let coords_right = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
            RelCoord { x: 1, y: 1 },
        ];
        let coords_down = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 2, y: 0 },
            RelCoord { x: 1, y: 1 },
        ];
        let coords_left = vec![
            RelCoord { x: -1, y: 1 },
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
        ];
        Self {
            name: "Teewee".to_string(),
            patterns: vec![
                Pattern { coords: coords_up },
                Pattern {
                    coords: coords_right,
                },
                Pattern {
                    coords: coords_down,
                },
                Pattern {
                    coords: coords_left,
                },
            ],
        }
    }

    #[allow(dead_code)]
    pub fn blue_ricky() -> Self {
        let coords_upright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
            RelCoord { x: -1, y: 2 },
        ];
        let coords_right = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 1, y: 1 },
            RelCoord { x: 2, y: 1 },
        ];
        let coords_downright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
            RelCoord { x: 1, y: 0 },
        ];
        let coords_left = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 2, y: 0 },
            RelCoord { x: 2, y: 1 },
        ];
        Self {
            name: "Blue Ricky".to_string(),
            patterns: vec![
                Pattern {
                    coords: coords_upright,
                },
                Pattern {
                    coords: coords_right,
                },
                Pattern {
                    coords: coords_downright,
                },
                Pattern {
                    coords: coords_left,
                },
            ],
        }
    }

    #[allow(dead_code)]
    pub fn orange_ricky() -> Self {
        let coords_upright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 2 },
            RelCoord { x: 1, y: 2 },
        ];
        let coords_right = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 2, y: 0 },
            RelCoord { x: 0, y: 1 },
        ];
        let coords_downright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 1, y: 1 },
            RelCoord { x: 1, y: 2 },
        ];
        let coords_left = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: -1, y: 1 },
            RelCoord { x: -2, y: 1 },
        ];
        Self {
            name: "Orange Ricky".to_string(),
            patterns: vec![
                Pattern {
                    coords: coords_upright,
                },
                Pattern {
                    coords: coords_right,
                },
                Pattern {
                    coords: coords_downright,
                },
                Pattern {
                    coords: coords_left,
                },
            ],
        }
    }

    #[allow(dead_code)]
    pub fn smashboy() -> Self {
        let coords = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 1, y: 1 },
        ];
        Self {
            name: "Smashboy".to_string(),
            patterns: vec![Pattern { coords }],
        }
    }

    #[allow(dead_code)]
    pub fn cleveland_z() -> Self {
        let flat = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
            RelCoord { x: 1, y: 1 },
            RelCoord { x: 2, y: 1 },
        ];
        let upright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: -1, y: 1 },
            RelCoord { x: -1, y: 2 },
        ];
        Self {
            name: "Cleveland Z".to_string(),
            patterns: vec![Pattern { coords: flat }, Pattern { coords: upright }],
        }
    }

    #[allow(dead_code)]
    pub fn rhode_island_z() -> Self {
        let flat = vec![
            RelCoord { x: -1, y: 1 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 1, y: 0 },
        ];
        let upright = vec![
            RelCoord { x: 0, y: 0 },
            RelCoord { x: 0, y: 1 },
            RelCoord { x: 1, y: 1 },
            RelCoord { x: 1, y: 2 },
        ];
        Self {
            name: "Rhode Island Z".to_string(),
            patterns: vec![Pattern { coords: flat }, Pattern { coords: upright }],
        }
    }
}

impl fmt::Display for Piece {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}:\n\n", self.name.red())?;
        for pattern in self.patterns.iter() {
            write!(f, "{}\n", pattern)?;
        }
        Ok(())
    }
}
