use crate::pattern::AbsCoord;
use crate::solve;
use crate::Field;
use crate::Piece;

#[test]
fn test_add_pattern() {
    let mut field = Field::new(3, 3);
    assert_eq!(
        field.add_pattern(&AbsCoord { x: 0, y: 0 }, &Piece::blue_ricky().patterns[0]),
        false
    );
    assert_eq!(
        field.add_pattern(&AbsCoord { x: 0, y: 0 }, &Piece::blue_ricky().patterns[1]),
        true
    );
}

#[test]
fn test_solve_no_pieces() {
    let field = Field::new(5, 5);
    let mut pieces = vec![];
    let result = solve(field, &mut pieces);
    assert!(result.is_none());
}

#[test]
fn test_solve_no_space() {
    let field = Field::new(0, 0);
    let mut pieces = vec![Piece::smashboy()];
    let result = solve(field, &mut pieces);
    assert!(result.is_none());
}

#[test]
fn test_solve_base_case() {
    let field = Field::new(0, 0);
    let mut pieces = vec![];
    let result = solve(field, &mut pieces);
    assert!(result.is_some());
}

#[test]
fn test_solve_four_smashboys() {
    let field = Field::new(4, 4);
    let mut pieces = vec![
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
    ];
    let result = solve(field, &mut pieces);
    assert!(result.is_some());
}

#[test]
fn test_two_two_one() {
    let field = Field::new(4, 5);
    let mut pieces = vec![
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::blue_ricky(),
        Piece::blue_ricky(),
        Piece::hero(),
    ];
    let result = solve(field, &mut pieces);
    assert!(result.is_some());
}

#[test]
fn test_two_two_one_fail() {
    let field = Field::new(4, 5);
    let mut pieces = vec![
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::blue_ricky(),
        Piece::orange_ricky(),
        Piece::hero(),
    ];
    let result = solve(field, &mut pieces);
    assert!(result.is_some());
}

#[test]
fn test_eight_seven() {
    let field = Field::new(8, 7);
    let mut pieces = vec![
        Piece::hero(), // ----
        Piece::hero(), // ----
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::smashboy(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::teewee(),
        Piece::cleveland_z(),    // -_
        Piece::rhode_island_z(), // _-
        Piece::orange_ricky(),   // ___|
        Piece::blue_ricky(),     // |___
    ];
    let result = solve(field, &mut pieces);
    assert!(result.is_some());
}
